import 'dart:collection';

import 'package:meta/meta.dart';
import 'package:miodi/core/consts.dart';

import 'module.dart';
import 'providers.dart';
import '../error/errors.dart';

class MiodiComponent {
  final Map<String, Map<Type, Provider>> factoryConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> singletonConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> customConfiguration = HashMap();

  @mustCallSuper
  MiodiComponent(this.modules) {
    modules.forEach((module) {
      factoryConfiguration.addAll(module.factoryConfiguration);
    });
    modules.forEach((module) {
      singletonConfiguration.addAll(module.singletonConfiguration);
    });
    modules.forEach((module) {
      customConfiguration.addAll(module.customConfiguration);
    });
  }

  final List<MiodiModule> modules;

  T get<T>({String name = defaultProviderName}) {
    if (singletonConfiguration[name].containsKey(T))
      return singletonConfiguration[name][T].provide() as T;
    if (factoryConfiguration[name].containsKey(T))
      return factoryConfiguration[name][T].provide() as T;
    if (customConfiguration[name].containsKey(T))
      return customConfiguration[name][T].provide() as T;
    throw NotFoundProviderException(T);
  }
}