import 'dart:async';
import 'dart:collection';

import 'package:miodi/core/consts.dart';

import '../error/errors.dart';
import 'component.dart';
import 'providers.dart';

class MiodiConfiguration {
  final Map<String, Map<Type, Provider>> _factoryConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> _singletonConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> _customConfiguration = HashMap();

  MiodiConfiguration(List<MiodiComponent> components) {
    components.forEach((component) {
      _factoryConfiguration.addAll(component.factoryConfiguration);
      _singletonConfiguration.addAll(component.singletonConfiguration);
      _customConfiguration.addAll(component.customConfiguration);
    });
  }

  T provide<T>({String name = defaultProviderName}) {
    if (_singletonConfiguration[name]!= null && _singletonConfiguration[name].containsKey(T)){
      return _singletonConfiguration[name][T].provide();
    }
    if (_factoryConfiguration[name]!= null && _factoryConfiguration[name].containsKey(T)){
      return _factoryConfiguration[name][T].provide();
    }
    if (_customConfiguration[name]!= null && _customConfiguration.containsKey(T)){
      return _customConfiguration[name][T].provide();
    }
    throw NotFoundProviderException(T);
  }

  Future<T> provideAsync<T>({String name = defaultProviderName}) async {
    return provide<T>(name: name);
  }
}