import 'dart:collection';

import 'providers.dart';

class MiodiModule {
  final Map<String, Map<Type, Provider>> factoryConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> singletonConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> customConfiguration = HashMap();

  MiodiModule(this.providers) {
    providers.forEach((provider) {
      final name = provider.name;
      if (provider is SingletonProvider) {
        if (!singletonConfiguration.containsKey(name))
          singletonConfiguration.addAll({name: HashMap()});
        _addProviderToConfiguration(provider, singletonConfiguration[name]);
      } else if (provider is FactoryProvider) {
        if (!factoryConfiguration.containsKey(name))
          factoryConfiguration.addAll({name: HashMap()});
        _addProviderToConfiguration(provider, factoryConfiguration[name]);
      } else {
        if (!customConfiguration.containsKey(name))
          customConfiguration.addAll({name: HashMap()});
        _addProviderToConfiguration(provider, customConfiguration[name]);
      }
    });
  }

  void _addProviderToConfiguration(Provider provider, Map<Type, Provider> configuration) {
    if (configuration.containsKey(provider.provideType)) {
      configuration[provider.provideType] = provider;
    } else
      configuration.addAll({provider.provideType: provider});
  }

  final List<Provider> providers;
}