library miodi;

import 'dart:async';
import 'package:miodi/core/consts.dart';

import 'core/component.dart';
import 'core/configuration.dart';
import 'core/module.dart';
import 'core/providers.dart';
import 'error/errors.dart';

export 'core/component.dart';
export 'core/module.dart';
export 'core/providers.dart';

Provider<T> singleton<T>(Type T, Creator<T> creator, {String name = defaultProviderName, bool initOnStart = false}) =>
    SingletonProvider(T, creator, name: name, initOnStart: initOnStart);

Provider<T> factory<T>(Type T, Creator<T> creator, {String name = defaultProviderName}) => FactoryProvider(T, creator, name: name);

MiodiModule module(List<Provider> providers) => MiodiModule(providers);

MiodiComponent component(List<MiodiModule> modules) => MiodiComponent(modules);

T inject<T>([String name]) => Miodi.inject(name);

Future<T> injectAsync<T>([String name]) => Miodi.injectAsync(name);

class Miodi {
  final MiodiConfiguration _configuration;
  static Miodi _instance;

  Miodi._(this._configuration);

  static void init(List<MiodiComponent> components) {
    if (_instance != null) return;
    _instance = Miodi._(MiodiConfiguration(components));
  }

  static T inject<T>([String name]) {
    if (_instance == null) throw NotInitializeMiodiException();
    if (name == null) {
      return _instance._configuration.provide();
    } else return _instance._configuration.provide(name: name);
  }


  static Future<T> injectAsync<T>([String name]) async {
    return inject<T>(name);
  }
}
